Class: Curl::Multi

Inherits:
Object
  • Object
show all
Defined in:
lib/curl/multi.rb,
ext/curb_multi.c

Defined Under Namespace

Classes: DownloadError

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initializeObject

Instance methods



384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
# File 'ext/curb_multi.c', line 384

static VALUE ruby_curl_multi_initialize(VALUE self) {
  ruby_curl_multi *rbcm;

  TypedData_Get_Struct(self, ruby_curl_multi, &ruby_curl_multi_data_type, rbcm);

  ruby_curl_multi_init(rbcm);

  /*
   * The mark routine will be called by the garbage collector during its ``mark'' phase.
   * If your structure references other Ruby objects, then your mark function needs to
   * identify these objects using rb_gc_mark(value). If the structure doesn't reference
   * other Ruby objects, you can simply pass 0 as a function pointer.
   */
  return self;
}

Class Method Details

.Curl::Multi.autocloseObject

Get the global default autoclose setting for all Curl::Multi Handles.



444
445
446
# File 'ext/curb_multi.c', line 444

VALUE ruby_curl_multi_get_autoclose(VALUE klass) {
  return cCurlMutiAutoClose == 1 ? Qtrue : Qfalse;
}

.Curl::Multi.autoclose( = true) ⇒ true

Automatically close open connections after each request. Otherwise, the connection will remain open for reuse until the next GC

Returns:

  • (true)


432
433
434
435
# File 'ext/curb_multi.c', line 432

VALUE ruby_curl_multi_set_autoclose(VALUE klass, VALUE onoff) {
  cCurlMutiAutoClose = ((onoff == Qtrue) ? 1 : 0);
  return onoff;
}

.Curl::Multi.default_timeout( = 4) ⇒ 4

Get the global default time out for all Curl::Multi Handles.

Returns:

  • (4)


420
421
422
# File 'ext/curb_multi.c', line 420

VALUE ruby_curl_multi_get_default_timeout(VALUE klass) {
  return LONG2NUM(cCurlMutiDefaulttimeout);
}

.Curl::Multi.default_timeout( = 4) ⇒ 4

Set the global default time out for all Curl::Multi Handles. This value is used when libcurl cannot determine a timeout value when calling curl_multi_timeout.

Returns:

  • (4)


408
409
410
411
# File 'ext/curb_multi.c', line 408

VALUE ruby_curl_multi_set_default_timeout(VALUE klass, VALUE timeout) {
  cCurlMutiDefaulttimeout = NUM2LONG(timeout);
  return timeout;
}

.download(urls, easy_options = {}, multi_options = {}, download_paths = nil, &blk) ⇒ Object

call-seq:

Curl::Multi.download(){|c|}

will create 2 new files file1.txt and file2.txt

2 files will be opened, and remain open until the call completes

when using the :post or :put method, urls should be a hash, including the individual post fields per post



191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
# File 'lib/curl/multi.rb', line 191

def download(urls,easy_options={},multi_options={},download_paths=nil,&blk)
  errors = []
  procs = []
  files = []
  urls_with_config = []
  url_to_download_paths = {}

  urls.each_with_index do|urlcfg,i|
    if urlcfg.is_a?(Hash)
      url = url[:url]
    else
      url = urlcfg
    end

    if download_paths and download_paths[i]
      download_path = download_paths[i]
    else
      download_path = File.basename(url)
    end

    file = lambda do|dp|
      file = File.open(dp,"wb")
      procs << (lambda {|data| file.write data; data.size })
      files << file
      file
    end.call(download_path)

    if urlcfg.is_a?(Hash)
      urls_with_config << urlcfg.merge({:on_body => procs.last}.merge(easy_options))
    else
      urls_with_config << {:url => url, :on_body => procs.last, :method => :get}.merge(easy_options)
    end
    url_to_download_paths[url] = {:path => download_path, :file => file} # store for later
  end

  if blk
    # when injecting the block, ensure file is closed before yielding
    Curl::Multi.http(urls_with_config, multi_options) do |c,code,method|
      info = url_to_download_paths[c.url]
      begin
        file = info[:file]
        files.reject!{|f| f == file }
        file.close
      rescue => e
        errors << e
      end
      blk.call(c,info[:path])
    end
  else
    Curl::Multi.http(urls_with_config, multi_options)
  end

ensure
  files.each {|f|
    begin
      f.close
    rescue => e
      errors << e
    end
  }
  if errors.any?
    de = Curl::Multi::DownloadError.new
    de.errors = errors
    raise de
  end
end

.get(urls, easy_options = {}, multi_options = {}, &blk) ⇒ Object

call-seq:

Curl::Multi.get(['url1','url2','url3','url4','url5'], :follow_location => true) do|easy|
  easy
end

Blocking call to fetch multiple url’s in parallel.



14
15
16
17
18
19
20
# File 'lib/curl/multi.rb', line 14

def get(urls, easy_options={}, multi_options={}, &blk)
  url_confs = []
  urls.each do|url|
    url_confs << {:url => url, :method => :get}.merge(easy_options)
  end
  self.http(url_confs, multi_options) {|c,code,method| blk.call(c) if blk }
end

.http(urls_with_config, multi_options = {}, &blk) ⇒ Object

call-seq:

Curl::Multi.http( [

{ :url => 'url1', :method => :post,
  :post_fields => {'field1' => 'value1', 'field2' => 'value2'} },
{ :url => 'url2', :method => :get,
  :follow_location => true, :max_redirects => 3 },
{ :url => 'url3', :method => :put, :put_data => File.open('file.txt','rb') },
{ :url => 'url4', :method => :head }

], => Curl::CURLPIPE_HTTP1)

Blocking call to issue multiple HTTP requests with varying verb’s.

urls_with_config: is a hash of url’s pointing to the easy handle options as well as the special option :method, that can by one of [:get, :post, :put, :delete, :head], when no verb is provided e.g. :method => nil -> GET is used multi_options: options for the multi handle blk: a callback, that yeilds when a handle is completed



88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
# File 'lib/curl/multi.rb', line 88

def http(urls_with_config, multi_options={}, &blk)
  m = Curl::Multi.new

  # maintain a sane number of easy handles
  multi_options[:max_connects] = max_connects = multi_options.key?(:max_connects) ? multi_options[:max_connects] : 10

  free_handles = [] # keep a list of free easy handles

  # configure the multi handle
  multi_options.each { |k,v| m.send("#{k}=", v) }
  callbacks = [:on_progress,:on_debug,:on_failure,:on_success,:on_redirect,:on_missing,:on_body,:on_header]

  add_free_handle = proc do|conf, easy|
    c       = conf.dup # avoid being destructive to input
    url     = c.delete(:url)
    method  = c.delete(:method)
    headers = c.delete(:headers)

    easy    = Curl::Easy.new if easy.nil?

    easy.url = url

    # assign callbacks
    callbacks.each do |cb|
      cbproc = c.delete(cb)
      easy.send(cb,&cbproc) if cbproc
    end

    case method
    when :post
      fields = c.delete(:post_fields)
      # set the post post using the url fields
      easy.post_body = fields.map{|f,k| "#{easy.escape(f)}=#{easy.escape(k)}"}.join('&')
    when :put
      easy.put_data = c.delete(:put_data)
    when :head
      easy.head = true
    when :delete
      easy.delete = true
    when :get
    else
      # XXX: nil is treated like a GET
    end

    # headers is a special key
    headers.each {|k,v| easy.headers[k] = v } if headers

    #
    # use the remaining options as specific configuration to the easy handle
    # bad options should raise an undefined method error
    #
    c.each { |k,v| easy.send("#{k}=",v) }

    easy.on_complete {|curl|
      free_handles << curl
      blk.call(curl,curl.response_code,method) if blk
    }
    m.add(easy)
  end

  max_connects.times do
    conf = urls_with_config.pop
    add_free_handle.call(conf, nil) if conf
    break if urls_with_config.empty?
  end

  consume_free_handles = proc do
    # as we idle consume free handles
    if urls_with_config.size > 0 && free_handles.size > 0
      easy = free_handles.pop
      conf = urls_with_config.pop
      add_free_handle.call(conf, easy) if conf
    end
  end

  begin
    if urls_with_config.empty?
      m.perform
    else
      until urls_with_config.empty?
        m.perform do
          consume_free_handles.call
        end
        consume_free_handles.call
      end
      free_handles = nil
    end
  ensure
    m.close
  end

end

.post(urls_with_config, easy_options = {}, multi_options = {}, &blk) ⇒ Object

call-seq:

Curl::Multi.post([{:url => 'url1', :post_fields => {'field1' => 'value1', 'field2' => 'value2'}},
                  {:url => 'url2', :post_fields => {'field1' => 'value1', 'field2' => 'value2'}},
                  {:url => 'url3', :post_fields => {'field1' => 'value1', 'field2' => 'value2'}}],
                 { :follow_location => true, :multipart_form_post => true },
                 {:pipeline => Curl::CURLPIPE_HTTP1}) do|easy|
  easy_handle_on_request_complete
end

Blocking call to POST multiple form’s in parallel.

urls_with_config: is a hash of url’s pointing to the postfields to send easy_options: are a set of common options to set on all easy handles multi_options: options to set on the Curl::Multi handle



38
39
40
41
42
43
44
# File 'lib/curl/multi.rb', line 38

def post(urls_with_config, easy_options={}, multi_options={}, &blk)
  url_confs = []
  urls_with_config.each do|uconf|
    url_confs << uconf.merge(:method => :post).merge(easy_options)
  end
  self.http(url_confs, multi_options) {|c,code,method| blk.call(c) }
end

.put(urls_with_config, easy_options = {}, multi_options = {}, &blk) ⇒ Object

call-seq:

Curl::Multi.put([{:url => 'url1', :put_data => "some message"},
                 {:url => 'url2', :put_data => IO.read('filepath')},
                 {:url => 'url3', :put_data => "maybe another string or socket?"],
                 {:follow_location => true},
                 {:pipeline => Curl::CURLPIPE_HTTP1}) do|easy|
  easy_handle_on_request_complete
end

Blocking call to POST multiple form’s in parallel.

urls_with_config: is a hash of url’s pointing to the postfields to send easy_options: are a set of common options to set on all easy handles multi_options: options to set on the Curl::Multi handle



62
63
64
65
66
67
68
# File 'lib/curl/multi.rb', line 62

def put(urls_with_config, easy_options={}, multi_options={}, &blk)
  url_confs = []
  urls_with_config.each do|uconf|
    url_confs << uconf.merge(:method => :put).merge(easy_options)
  end
  self.http(url_confs, multi_options) {|c,code,method| blk.call(c) }
end

Instance Method Details

#_add(easy) ⇒ Object

multi = Curl::Multi.new easy = Curl::Easy.new(‘url’)

multi.add(easy)

Add an easy handle to the multi stack



536
537
538
539
540
541
542
543
544
545
546
547
548
549
550
551
552
553
554
555
556
557
558
559
560
561
562
563
564
565
566
567
568
569
570
571
572
573
574
575
576
577
# File 'ext/curb_multi.c', line 536

VALUE ruby_curl_multi_add(VALUE self, VALUE easy) {
  CURLMcode mcode;
  ruby_curl_easy *rbce;
  ruby_curl_multi *rbcm;

  TypedData_Get_Struct(self, ruby_curl_multi, &ruby_curl_multi_data_type, rbcm);
  TypedData_Get_Struct(easy, ruby_curl_easy, &ruby_curl_easy_data_type, rbce);
  ruby_curl_multi_ensure_handle(rbcm);

  /* setup the easy handle */
  ruby_curl_easy_setup( rbce );

  mcode = curl_multi_add_handle(rbcm->handle, rbce->curl);
  if (mcode != CURLM_CALL_MULTI_PERFORM && mcode != CURLM_OK) {
    ruby_curl_easy_cleanup(easy, rbce);

    raise_curl_multi_error_exception(mcode);
  }

  rbcm->active++;

  /* Increase the running count, so that the perform loop keeps running.
   * If this number is not correct, the next call to curl_multi_perform will correct it. */
  rbcm->running++;

  if (!rbcm->attached) {
    rbcm->attached = st_init_numtable();
    if (!rbcm->attached) {
      curl_multi_remove_handle(rbcm->handle, rbce->curl);
      ruby_curl_easy_cleanup(easy, rbce);
      rb_raise(rb_eNoMemError, "Failed to allocate multi attachment table");
    }
  }

  rbce->multi_attachment_generation++;
  st_insert(rbcm->attached, (st_data_t)rbce, (st_data_t)easy);

  /* track a reference to associated multi handle */
  rbce->multi = self;

  return self;
}

#_closeObject

multi.close after closing the multi handle all connections will be closed and the handle will no longer be usable



1740
1741
1742
1743
1744
1745
1746
1747
1748
1749
1750
1751
1752
1753
1754
1755
# File 'ext/curb_multi.c', line 1740

VALUE ruby_curl_multi_close(VALUE self) {
  ruby_curl_multi *rbcm;
  TypedData_Get_Struct(self, ruby_curl_multi, &ruby_curl_multi_data_type, rbcm);
  rb_curl_multi_detach_all(rbcm);

  if (rbcm->handle) {
    curl_multi_cleanup(rbcm->handle);
    rbcm->handle = NULL;
  }

  rbcm->active = 0;
  rbcm->running = 0;
  clear_multi_deferred_exception_if_any(self);
  clear_multi_deferred_exception_source_id_if_any(self);
  return self;
}

#_remove(rb_easy_handle) ⇒ Object

multi = Curl::Multi.new easy = Curl::Easy.new(‘url’)

multi.add(easy)

# sometime later multi.remove(easy)

Remove an easy handle from a multi stack.

Will raise an exception if the easy handle is not found



593
594
595
596
597
598
599
600
601
# File 'ext/curb_multi.c', line 593

VALUE ruby_curl_multi_remove(VALUE self, VALUE rb_easy_handle) {
  ruby_curl_multi *rbcm;

  TypedData_Get_Struct(self, ruby_curl_multi, &ruby_curl_multi_data_type, rbcm);

  rb_curl_multi_remove(rbcm, rb_easy_handle);

  return self;
}

#add(easy) ⇒ Object



312
313
314
315
316
317
318
319
320
321
# File 'lib/curl/multi.rb', line 312

def add(easy)
  return self if requests[easy.object_id]
  # Once a deferred callback exception is pending, Multi#perform is
  # draining existing transfers only and must not start replacement work.
  return self if instance_variable_defined?(:@__curb_deferred_exception)
  _add(easy)
  __unregister_idle_easy_reference(easy)
  requests[easy.object_id] = easy
  self
end

#cancel!Object



259
260
261
262
263
# File 'lib/curl/multi.rb', line 259

def cancel!
  requests.each do |_,easy|
    remove(easy)
  end
end

#closeObject



330
331
332
# File 'lib/curl/multi.rb', line 330

def close
  __close(true)
end

#idle?Boolean

Returns:

  • (Boolean)


265
266
267
# File 'lib/curl/multi.rb', line 265

def idle?
  requests.empty?
end

#max_connects=(count) ⇒ Object

multi = Curl::Multi.new multi.max_connects = 800

Set the max connections in the cache for a multi handle



461
462
463
464
465
466
467
468
469
470
471
472
# File 'ext/curb_multi.c', line 461

static VALUE ruby_curl_multi_max_connects(VALUE self, VALUE count) {
#ifdef HAVE_CURLMOPT_MAXCONNECTS
  ruby_curl_multi *rbcm;

  TypedData_Get_Struct(self, ruby_curl_multi, &ruby_curl_multi_data_type, rbcm);
  ruby_curl_multi_ensure_handle(rbcm);

  curl_multi_setopt(rbcm->handle, CURLMOPT_MAXCONNECTS, NUM2LONG(count));
#endif

  return count;
}

#max_host_connections=(count) ⇒ Object

multi = Curl::Multi.new multi.max_host_connections = 1

Set the max number of connections per host



481
482
483
484
485
486
487
488
489
490
491
492
# File 'ext/curb_multi.c', line 481

static VALUE ruby_curl_multi_max_host_connections(VALUE self, VALUE count) {
#ifdef HAVE_CURLMOPT_MAX_HOST_CONNECTIONS
  ruby_curl_multi *rbcm;

  TypedData_Get_Struct(self, ruby_curl_multi, &ruby_curl_multi_data_type, rbcm);
  ruby_curl_multi_ensure_handle(rbcm);

  curl_multi_setopt(rbcm->handle, CURLMOPT_MAX_HOST_CONNECTIONS, NUM2LONG(count));
#endif

  return count;
}

#perform(*args) ⇒ Object

multi = Curl::Multi.new easy1 = Curl::Easy.new(‘url’) easy2 = Curl::Easy.new(‘url’)

multi.add(easy1) multi.add(easy2)

multi.perform do

# while idle other code my execute here

end

Run multi handles, looping selecting when data can be transfered



1513
1514
1515
1516
1517
1518
1519
1520
1521
1522
1523
1524
1525
1526
1527
1528
1529
1530
1531
1532
1533
1534
1535
1536
1537
1538
1539
1540
1541
1542
1543
1544
1545
1546
1547
1548
1549
1550
1551
1552
1553
1554
1555
1556
1557
1558
1559
1560
1561
1562
1563
1564
1565
1566
1567
1568
1569
1570
1571
1572
1573
1574
1575
1576
1577
1578
1579
1580
1581
1582
1583
1584
1585
1586
1587
1588
1589
1590
1591
1592
1593
1594
1595
1596
1597
1598
1599
1600
1601
1602
1603
1604
1605
1606
1607
1608
1609
1610
1611
1612
1613
1614
1615
1616
1617
1618
1619
1620
1621
1622
1623
1624
1625
1626
1627
1628
1629
1630
1631
1632
1633
1634
1635
1636
1637
1638
1639
1640
1641
1642
1643
1644
1645
1646
1647
1648
1649
1650
1651
1652
1653
1654
1655
1656
1657
1658
1659
1660
1661
1662
1663
1664
1665
1666
1667
1668
1669
1670
1671
1672
1673
1674
1675
1676
1677
1678
1679
1680
1681
1682
1683
1684
1685
1686
1687
1688
1689
1690
1691
1692
1693
1694
1695
1696
1697
1698
1699
1700
1701
1702
1703
1704
1705
1706
1707
1708
1709
1710
1711
1712
1713
1714
1715
1716
1717
1718
1719
1720
1721
1722
1723
1724
1725
1726
1727
1728
1729
1730
1731
# File 'ext/curb_multi.c', line 1513

VALUE ruby_curl_multi_perform(int argc, VALUE *argv, VALUE self) {
  CURLMcode mcode;
  ruby_curl_multi *rbcm;
  int maxfd, rc = -1;
  fd_set fdread, fdwrite, fdexcep;
#ifdef _WIN32
  fd_set crt_fdread, crt_fdwrite, crt_fdexcep;
#endif
  long timeout_milliseconds;
  struct timeval tv = {0, 0};
  struct timeval tv_100ms = {0, 100000};
  VALUE block = Qnil;
#if !defined(HAVE_RB_THREAD_FD_SELECT) && (defined(HAVE_RB_THREAD_BLOCKING_REGION) || defined(HAVE_RB_THREAD_CALL_WITHOUT_GVL))
  struct _select_set fdset_args;
#endif

  rb_scan_args(argc, argv, "0&", &block);

  TypedData_Get_Struct(self, ruby_curl_multi, &ruby_curl_multi_data_type, rbcm);
  ruby_curl_multi_ensure_handle(rbcm);
  if (!rb_ivar_defined(self, id_deferred_exception_ivar)) {
    clear_multi_deferred_exception_source_id_if_any(self);
  }

  timeout_milliseconds = cCurlMutiDefaulttimeout;

  // Run curl_multi_perform for the first time to get the ball rolling
  rb_curl_multi_run( self, rbcm->handle, &(rbcm->running) );

  // Check the easy handles for new messages one more time before yielding
  // control to passed ruby block.
  //
  // This call will block until all queued messages are processed and if any
  // handle completed the transfer we will run the on_complete callback here too.
  rb_curl_multi_read_info( self, rbcm->handle );

  // There are no more messages to handle by curl and we can run the ruby block
  // passed to perform method.
  // When the block completes curl will resume.
  rb_curl_multi_yield_if_given(self, block);

  do {
    while (rbcm->running) {
#ifdef HAVE_CURL_MULTI_TIMEOUT
      /* get the curl suggested time out */
      mcode = curl_multi_timeout(rbcm->handle, &timeout_milliseconds);
      if (mcode != CURLM_OK) {
        raise_curl_multi_error_exception(mcode);
      }
#else
      /* libcurl doesn't have a timeout method defined, initialize to -1 we'll pick up the default later */
      timeout_milliseconds = -1;
#endif

      if (timeout_milliseconds == 0) { /* no delay */
        rb_curl_multi_run( self, rbcm->handle, &(rbcm->running) );
        rb_curl_multi_read_info( self, rbcm->handle );
        rb_curl_multi_yield_if_given(self, block);
#if defined(HAVE_RB_FIBER_SCHEDULER_CURRENT)
        if (rb_fiber_scheduler_current() != Qnil) {
          rb_thread_schedule();
        }
#endif
        continue;
      }

      if (timeout_milliseconds < 0 || timeout_milliseconds > cCurlMutiDefaulttimeout) {
        timeout_milliseconds = cCurlMutiDefaulttimeout; /* libcurl doesn't know how long to wait, use a default timeout */
                                                        /* or buggy versions libcurl sometimes reports huge timeouts... let's cap it */
      }

#if defined(HAVE_CURL_MULTI_WAIT) && !defined(HAVE_RB_THREAD_FD_SELECT)
      {
        struct wait_args wait_args;
        wait_args.handle     = rbcm->handle;
        wait_args.timeout_ms = timeout_milliseconds;
        wait_args.numfds     = 0;
        /*
         * When a Fiber scheduler is available (Ruby >= 3.x), rb_thread_fd_select
         * integrates with it. If we have rb_thread_fd_select available at build
         * time, we avoid curl_multi_wait entirely (see preprocessor guard above)
         * and use the fdset branch below. Otherwise, we use curl_multi_wait and
         * release the GVL so Ruby threads can continue to run.
         */
        CURLMcode wait_rc;
#if defined(HAVE_RB_THREAD_CALL_WITHOUT_GVL)
        wait_rc = (CURLMcode)(intptr_t)rb_thread_call_without_gvl(
          curl_multi_wait_wrapper, &wait_args, RUBY_UBF_IO, NULL
        );
#else
        wait_rc = curl_multi_wait(rbcm->handle, NULL, 0, timeout_milliseconds, &wait_args.numfds);
#endif
        if (wait_rc != CURLM_OK) {
          raise_curl_multi_error_exception(wait_rc);
        }
        if (wait_args.numfds == 0) {
#ifdef HAVE_RB_THREAD_FD_SELECT
          struct timeval tv_sleep = tv_100ms;
          /* Sleep in a scheduler-aware way. */
          rb_thread_fd_select(0, NULL, NULL, NULL, &tv_sleep);
#else
          rb_thread_wait_for(tv_100ms);
#endif
        }
        /* Process pending transfers after waiting */
        rb_curl_multi_run(self, rbcm->handle, &(rbcm->running));
        rb_curl_multi_read_info(self, rbcm->handle);
        rb_curl_multi_yield_if_given(self, block);
      }
#else

      tv.tv_sec  = 0; /* never wait longer than 1 second */
      tv.tv_usec = (int)(timeout_milliseconds * 1000); /* XXX: int is the right type for OSX, what about linux? */

      FD_ZERO(&fdread);
      FD_ZERO(&fdwrite);
      FD_ZERO(&fdexcep);

      /* load the fd sets from the multi handle */
      mcode = curl_multi_fdset(rbcm->handle, &fdread, &fdwrite, &fdexcep, &maxfd);
      if (mcode != CURLM_OK) {
        raise_curl_multi_error_exception(mcode);
      }

      if (maxfd == -1) {
        /* libcurl recommends sleeping for 100ms */
#ifdef HAVE_RB_THREAD_FD_SELECT
        struct timeval tv_sleep = tv_100ms;
        rb_thread_fd_select(0, NULL, NULL, NULL, &tv_sleep);
#else
        rb_thread_wait_for(tv_100ms);
#endif
        rb_curl_multi_run( self, rbcm->handle, &(rbcm->running) );
        rb_curl_multi_read_info( self, rbcm->handle );
        rb_curl_multi_yield_if_given(self, block);
        continue;
      }

#ifdef _WIN32
      create_crt_fd(&fdread, &crt_fdread);
      create_crt_fd(&fdwrite, &crt_fdwrite);
      create_crt_fd(&fdexcep, &crt_fdexcep);
#endif


#if !defined(HAVE_RB_THREAD_FD_SELECT) && (defined(HAVE_RB_THREAD_BLOCKING_REGION) || defined(HAVE_RB_THREAD_CALL_WITHOUT_GVL))
      fdset_args.maxfd = maxfd+1;
      fdset_args.fdread = &fdread;
      fdset_args.fdwrite = &fdwrite;
      fdset_args.fdexcep = &fdexcep;
      fdset_args.tv = &tv;
#endif

#ifdef HAVE_RB_THREAD_FD_SELECT
      /* Prefer scheduler-aware waiting when available. Build rb_fdset_t sets. */
      {
        rb_fdset_t rfds, wfds, efds;
        rb_fd_init(&rfds);
        rb_fd_init(&wfds);
        rb_fd_init(&efds);
#ifdef _WIN32
        /* On Windows, iterate explicit fd arrays for CRT fds. */
        int i;
        for (i = 0; i < crt_fdread.fd_count; i++) rb_fd_set(crt_fdread.fd_array[i], &rfds);
        for (i = 0; i < crt_fdwrite.fd_count; i++) rb_fd_set(crt_fdwrite.fd_array[i], &wfds);
        for (i = 0; i < crt_fdexcep.fd_count; i++) rb_fd_set(crt_fdexcep.fd_array[i], &efds);
        rc = rb_thread_fd_select(0, &rfds, &wfds, &efds, &tv);
#else
        int fd;
        for (fd = 0; fd <= maxfd; fd++) {
          if (FD_ISSET(fd, &fdread)) rb_fd_set(fd, &rfds);
          if (FD_ISSET(fd, &fdwrite)) rb_fd_set(fd, &wfds);
          if (FD_ISSET(fd, &fdexcep)) rb_fd_set(fd, &efds);
        }
        rc = rb_thread_fd_select(maxfd+1, &rfds, &wfds, &efds, &tv);
#endif
        rb_fd_term(&rfds);
        rb_fd_term(&wfds);
        rb_fd_term(&efds);
      }
#elif defined(HAVE_RB_THREAD_CALL_WITHOUT_GVL)
      rc = (int)(VALUE) rb_thread_call_without_gvl((void *(*)(void *))curb_select, &fdset_args, RUBY_UBF_IO, 0);
#elif HAVE_RB_THREAD_BLOCKING_REGION
      rc = rb_thread_blocking_region(curb_select, &fdset_args, RUBY_UBF_IO, 0);
#else
      rc = select(maxfd+1, &fdread, &fdwrite, &fdexcep, &tv);
#endif

#ifdef _WIN32
      cleanup_crt_fd(&fdread, &crt_fdread);
      cleanup_crt_fd(&fdwrite, &crt_fdwrite);
      cleanup_crt_fd(&fdexcep, &crt_fdexcep);
#endif

      switch(rc) {
      case -1:
        if(errno != EINTR) {
          rb_raise(rb_eRuntimeError, "select(): %s", strerror(errno));
          break;
        }
      case 0: /* timeout */
      default: /* action */
        rb_curl_multi_run( self, rbcm->handle, &(rbcm->running) );
        rb_curl_multi_read_info( self, rbcm->handle );
        rb_curl_multi_yield_if_given(self, block);
        break;
      }
#endif /* disabled curl_multi_wait: use fdsets */
    }

  } while( rbcm->running );

  rb_curl_multi_read_info( self, rbcm->handle );
  rb_curl_multi_yield_if_given(self, block);
  if (cCurlMutiAutoClose  == 1) {
    rb_funcall(self, rb_intern("_autoclose"), 0);
  }
  return Qtrue;
}

#pipeline=(method) ⇒ Object

multi = Curl::Multi.new multi.pipeline = true

Pass a long set to 1 for HTTP/1.1 pipelining, 2 for HTTP/2 multiplexing, or 0 to disable.

Enabling pipelining on a multi handle will make it attempt to perform HTTP Pipelining as

far as possible for transfers using this handle. This means that if you add a second request that can use an already existing connection, the second request will be “piped” on the same connection rather than being executed in parallel. (Added in 7.16.0, multiplex added in 7.43.0)



506
507
508
509
510
511
512
513
514
515
516
517
518
519
520
521
522
523
524
525
# File 'ext/curb_multi.c', line 506

static VALUE ruby_curl_multi_pipeline(VALUE self, VALUE method) {
#ifdef HAVE_CURLMOPT_PIPELINING
  ruby_curl_multi *rbcm;

  long value;

  if (method == Qtrue) {
    value = 1;
  } else if (method == Qfalse) {
    value  = 0;
  } else {
    value = NUM2LONG(method);
  } 

  TypedData_Get_Struct(self, ruby_curl_multi, &ruby_curl_multi_data_type, rbcm);
  ruby_curl_multi_ensure_handle(rbcm);
  curl_multi_setopt(rbcm->handle, CURLMOPT_PIPELINING, value);
#endif
  return method == Qtrue ? 1 : 0;
}

#remove(easy) ⇒ Object



323
324
325
326
327
328
# File 'lib/curl/multi.rb', line 323

def remove(easy)
  return self if !requests[easy.object_id]
  requests.delete(easy.object_id)
  _remove(easy)
  self
end

#requestsObject



269
270
271
# File 'lib/curl/multi.rb', line 269

def requests
  @requests ||= {}
end